<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins
 * @since      1.0.0
 *
 * @package    WooCommerce_Ultimate_Pricing
 * @subpackage WooCommerce_Ultimate_Pricing/admin
 */
class WooCommerce_Ultimate_Pricing_Admin extends WooCommerce_Ultimate_Pricing {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * Construct the Class
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://welaunch.io
	 * @param   [type]                       $plugin_name [description]
	 * @param   [type]                       $version     [description]
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->userRoles = array();

	}

    /**
     * Enqueue Admin Styles
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://welaunch.io/plugins/woocommerce-ultimate-pricing/
     * @return  boolean
     */
    public function enqueue_styles()
    {
		wp_enqueue_style( 'flatpickr',  plugin_dir_url( dirname( __FILE__ ) ) . 'assets/css/flatpickr.min.css', array(), '4.6.3', 'all' );
		wp_enqueue_style( $this->plugin_name.'-admin', plugin_dir_url( dirname( __FILE__ ) ) . 'assets/css/woocommerce-ultimate-pricing-admin.css', array(), $this->version);
    }

    /**
     * Enqueue Admin Scripts
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    http://www.welaunch.io
     * @return  boolean
     */
    public function enqueue_scripts()
    {
		wp_enqueue_script( 'flatpickr',  plugin_dir_url( dirname( __FILE__ ) ) . 'assets/js/flatpickr.min.js', array( 'jquery'), '4.6.3', true );
        wp_enqueue_script( $this->plugin_name.'-admin', plugin_dir_url( dirname( __FILE__ ) ) . 'assets/js/woocommerce-ultimate-pricing-admin.js', array('jquery', 'flatpickr'), $this->version, true);
    }


	/**
	 * Load Redux
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://welaunch.io
	 * @return  [type]                       [description]
	 */
	public function load_redux(){
	    if ( file_exists( plugin_dir_path( dirname( __FILE__ ) ) . 'config/options-init.php' ) ) {
	        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'config/options-init.php';
	    }
	}

	/**
	 * Init Admin
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://welaunch.io
	 * @return  [type]             [description]
	 */
	public function init()
	{
		global $woocommerce_ultimate_pricing_options;
		$this->options = $woocommerce_ultimate_pricing_options;

		if (!$this->get_option('enable')) {
			return false;
		}

		if(!$this->get_option('customUserRolePrices') && !$this->get_option('tieredPricingEnabled')) {
			return false;
		}

		global $wp_roles;
		if(isset($wp_roles->roles)) {
			$this->userRoles = array_keys($wp_roles->roles);
		}

		// Price per User Role
		add_action('woocommerce_product_options_general_product_data', array($this, 'custom_fields') );
		add_action('woocommerce_process_product_meta', array($this, 'save_custom_fields') );

		add_action('woocommerce_product_after_variable_attributes',array($this, 'custom_variation_fields'), 10, 3 ); 
		add_filter('woocommerce_save_product_variation', array($this,'save_product_variation'), 10, 2 );
	}

	public function custom_fields()
	{
		global $post;

		$product = wc_get_product($post->ID);
		if(!$product) {
			return;
		}

		$productId = $product->get_id();

		echo '<div class="woocommerce-ultimate-pricing">';

			if(!$this->get_option('customUserRolePricesDisableBackend') && !$product->is_type('variable')) {

				echo '<h2>' . esc_html__('User Role Pricing', 'woocommerce-ultimate-pricing') . '</h2>';
			
				$prices = get_post_meta($product->get_id(), 'woocommerce_ultimate_pricing_prices', true);	

				$i = 0;
				foreach ($this->userRoles as $userRole) {

					$value = "";
					if(isset($prices[$userRole]) && !empty($prices[$userRole])) {
						$value = $prices[$userRole];
					}
					
				    woocommerce_wp_text_input(
				        array(
				            'id' => 'woocommerce_ultimate_pricing_prices[' . $productId . '][' . $userRole . ']',
				            'placeholder' => ucfirst($userRole) . ' ' . esc_html__('Price', 'woocommerce-ultimate-pricing'),
				            'label' => ucfirst($userRole) . ' ' . esc_html__('Price', 'woocommerce-ultimate-pricing'),
							'data_type'     => 'price',
				            'value'	=> $value
				        )
			        );

			        $i++;
		        }
	        }

	        if($this->get_option('tieredPricingEnabled') && !$this->get_option('tieredPricingDisableBackend')) {

	        	echo '<hr><h2>' . esc_html__('Tiered Pricing', 'woocommerce-ultimate-pricing') . '</h2>';

	        	$tieredPricing = get_post_meta($productId, 'woocommerce_ultimate_pricing_tiered_pricing', true);
	        	$tieredPricingType = "percentage";
	        	$tieredPricingDiscounts = array();

	        	if(!empty($tieredPricing) && isset($tieredPricing['discounts']) && isset($tieredPricing['type'])) {
	        		$tieredPricingType = $tieredPricing['type'];
	        		$tieredPricingDiscounts = array_filter($tieredPricing['discounts']);
	        	}


				woocommerce_wp_select(
					array(
						'id'          => 'woocommerce_ultimate_pricing_tiered_pricing[' . $productId . '][type]',
						'value'       => $tieredPricingType,
						'label'       => __( 'Tiered Pricing Type', 'woocommerce-ultimate-pricing' ),
						'options'     => array(
							'percentage'  => __( 'Percentage Discount', 'woocommerce-ultimate-pricing' ),
							'fixed' => __( 'Fixed Discount', 'woocommerce-ultimate-pricing' ),
							'fixed_total' => __( 'Fixed Total', 'woocommerce-ultimate-pricing' ),
						),
						'desc_tip'    => 'true',
						'description' => __( 'Define if the tiered pricing discount should be percentage or fixed amount based.', 'woocommerce-ultimate-pricing' ),
					)
				);

				$index = 0;
				if(!empty($tieredPricingDiscounts)) {
					foreach ($tieredPricingDiscounts as $tieredPricing_discount_key => $tieredPricingDiscountValue) {
						$tieredPricingDiscountValue = explode('|', $tieredPricingDiscountValue);
						if(!isset($tieredPricingDiscountValue[0]) || !isset($tieredPricingDiscountValue[1])) {
							continue;
						}

						$tieredPricingQuantity = $tieredPricingDiscountValue[0];
						$tieredPricingPrice = $tieredPricingDiscountValue[1];

						if(!empty($tieredPricingQuantity) && !empty($tieredPricingPrice)) {

							echo '<div class="woocommerce-ultimate-pricing-tiered-pricing-row" data-index="' . $index . '">';

							    woocommerce_wp_text_input(
							        array(
							            'id' => 'woocommerce_ultimate_pricing_tiered_pricing[' . $productId . '][discounts][' . $index . '][quantity]',
							            'placeholder' =>  esc_html__('Quantity', 'woocommerce-ultimate-pricing'),
							            'label' =>  esc_html__('Quantity', 'woocommerce-ultimate-pricing'),
										'type'     => 'number',
										'wrapper_class' => 'woocommerce-tiered-pricing-row-left',
							            'custom_attributes' => array(
							                'step' => 'any',
							                'min' => '0'
							            ),
							            'value'	=> $tieredPricingQuantity
							        )
						        );

							    woocommerce_wp_text_input(
							        array(
							            'id' => 'woocommerce_ultimate_pricing_tiered_pricing[' . $productId . '][discounts][' . $index . '][discount]',
							            'placeholder' => esc_html__('Discount', 'woocommerce-ultimate-pricing'),
							            'label' => esc_html__('Discount', 'woocommerce-ultimate-pricing'),
										'type'     => 'number',
										'wrapper_class' => 'woocommerce-tiered-pricing-row-right',
							            'custom_attributes' => array(
							                'step' => 'any',
							                'min' => '0'
							            ),
							            'value'	=> $tieredPricingPrice
							        )
						        );

						        echo '<div class="woocommerce-ultimate-pricing-tiered-pricing-clear"></div>';

						    echo '</div>';

						    $index++;
						}
					}
				}



				echo '<div class="woocommerce-ultimate-pricing-tiered-pricing-row woocommerce-ultimate-pricing-tiered-pricing-to-copy" data-index="' . $index . '">';

				    woocommerce_wp_text_input(
				        array(
				            'id' => 'woocommerce_ultimate_pricing_tiered_pricing[' . $productId . '][discounts][' . $index . '][quantity]',
				            'placeholder' =>  esc_html__('Quantity', 'woocommerce-ultimate-pricing'),
				            'label' =>  esc_html__('Quantity', 'woocommerce-ultimate-pricing'),
							'type'     => 'number',
							'wrapper_class' => 'woocommerce-tiered-pricing-row-left',
				            'custom_attributes' => array(
				                'step' => 'any',
				                'min' => '0'
				            ),
				            'value'	=> '',
				        )
			        );

				    woocommerce_wp_text_input(
				        array(
				            'id' => 'woocommerce_ultimate_pricing_tiered_pricing[' . $productId . '][discounts][' . $index . '][discount]',
				            'placeholder' => esc_html__('Discount', 'woocommerce-ultimate-pricing'),
				            'label' => esc_html__('Discount', 'woocommerce-ultimate-pricing'),
							'type'     => 'number',
							'wrapper_class' => 'woocommerce-tiered-pricing-row-right',
				            'custom_attributes' => array(
				                'step' => 'any',
				                'min' => '0'
				            ),
				            'value'	=> '',
				        )
			        );

			        echo '<div class="woocommerce-ultimate-pricing-tiered-pricing-clear"></div>';

			    echo '</div>';

		        echo '<a href="#" class="button button-primary woocommerce-ultimate-pricing-tiered-pricing-add-row">' . esc_html__('Add Row', 'woocommerce-ultimate-pricing') . '</a>';
        	}

	     echo '</div>';

	}

	public function save_custom_fields($productId)
	{
		$product = wc_get_product($productId);

		if(isset($_POST['woocommerce_ultimate_pricing_prices'][$productId]) && $_POST['woocommerce_ultimate_pricing_prices'][$productId] !== "") {
		    $prices = isset($_POST['woocommerce_ultimate_pricing_prices'][$productId]) ? $_POST['woocommerce_ultimate_pricing_prices'][$productId] : '';
		    $product->update_meta_data('woocommerce_ultimate_pricing_prices', $prices);
		} else {
			$product->delete_meta_data('woocommerce_ultimate_pricing_prices');
		}

		if(isset($_POST['woocommerce_ultimate_pricing_tiered_pricing'][$productId]) && $_POST['woocommerce_ultimate_pricing_tiered_pricing'][$productId] !== "") {
		    $tieredPricing = $_POST['woocommerce_ultimate_pricing_tiered_pricing'][$productId];

		    if(isset($tieredPricing['type']) && isset($tieredPricing['discounts']) && !empty($tieredPricing['discounts'])) {
		    	$tieredPricing_type = $tieredPricing['type'];
		    	$tieredPricing_discounts = array_filter($tieredPricing['discounts']);

		    	$tmp = array();
		    	foreach ($tieredPricing_discounts as $tieredPricing_discount_key => $tieredPricing_discount_value) {
					if(!isset($tieredPricing_discount_value['quantity']) || !isset($tieredPricing_discount_value['discount'])) {
						continue;
					}

					$tieredPricingQuantity = $tieredPricing_discount_value['quantity'];
					$tieredPricingPrice = $tieredPricing_discount_value['discount'];

					if(!empty($tieredPricingQuantity) && !empty($tieredPricingPrice)) {
						$tmp[] = $tieredPricingQuantity . '|' . $tieredPricingPrice;
					}
		    	}

		    	$tieredPricing['discounts'] = $tmp;
		    	$product->update_meta_data('woocommerce_ultimate_pricing_tiered_pricing', $tieredPricing);
		    } else {
		    	$product->delete_meta_data('woocommerce_ultimate_pricing_tiered_pricing');
		    }
		    
		} else {
			$product->delete_meta_data('woocommerce_ultimate_pricing_tiered_pricing');
		}

		$product->save();
	}

	/**
	 * Add variation title backend field
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https:/welaunch.io
	 * @param   [type]             $loop           [description]
	 * @param   [type]             $variation_data [description]
	 * @param   [type]             $variation      [description]
	 */
	public function custom_variation_fields($loop, $variation_data, $variation)
	{
	    echo '<div class="woocommerce-ultimate-pricing">';

			if(!$this->get_option('customUserRolePricesDisableBackend')) {
			
				$prices = get_post_meta($variation->ID, 'woocommerce_ultimate_pricing_prices', true);	

				$i = 0;
				foreach ($this->userRoles as $userRole) {

					$value = "";
					if(isset($prices[$userRole]) && !empty($prices[$userRole])) {
						$value = $prices[$userRole];
					}

					$wrapper_class = 'form-row ';
					if($i % 2 == 0) {
						$wrapper_class .= 'form-row-first';
					} else {
						$wrapper_class .= 'form-row-last';
					}

				    woocommerce_wp_text_input(
				        array(
				            'id' => 'woocommerce_ultimate_pricing_prices[' . $variation->ID . '][' . $userRole . ']',
				            'placeholder' => ucfirst($userRole) . ' ' . esc_html__('Price', 'woocommerce-ultimate-pricing'),
				            'label' => ucfirst($userRole) . ' ' . esc_html__('Price', 'woocommerce-ultimate-pricing'),
							'data_type'     => 'price',
							'wrapper_class' => $wrapper_class,
				            'custom_attributes' => array(
				                'step' => 'any',
				                'min' => '0'
				            ),
				            'value'	=> $value
				        )
			        );

			        $i++;
		        }
	        }
        echo '</div>';
	}

	/**
	 * Save Variation Data (title)
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https:/welaunch.io
	 * @param   [type]             $variation_id [description]
	 * @param   [type]             $i            [description]
	 * @return  [type]                           [description]
	 */
	public function save_product_variation($variation_id, $i)
	{
		if(empty($variation_id)) {
			return;
		}

		$product = wc_get_product($variation_id);

		if(!$this->get_option('customUserRolePricesDisableBackend')) {
			if(isset($_POST['woocommerce_ultimate_pricing_prices'][$variation_id]) && $_POST['woocommerce_ultimate_pricing_prices'][$variation_id] !== "") {
			    $prices = isset($_POST['woocommerce_ultimate_pricing_prices'][$variation_id]) ? $_POST['woocommerce_ultimate_pricing_prices'][$variation_id] : '';
			    $product->update_meta_data('woocommerce_ultimate_pricing_prices', $prices);
			} else {
				$product->delete_meta_data('woocommerce_ultimate_pricing_prices');
			}
		}

		$product->save();
	}
}		