<?php
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Writer\Xls;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;

class WooCommerce_Ultimate_Pricing_Importer extends WooCommerce_Ultimate_Pricing
{
    protected $plugin_name;
    protected $version;

    public $notice;

    /**
     * Construct Price Locator Admin Class
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://welaunch.io/plugins
     * @param   string                         $plugin_name
     * @param   string                         $version    
     */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->notice = "";
        $this->file = "";
        $this->rows = 0;
        $this->columns = 0;
    }

    public function init()
    {
        global $woocommerce_ultimate_pricing_options;
        $this->options = $woocommerce_ultimate_pricing_options;

        global $wp_roles;
        if(isset($wp_roles->roles)) {
            $this->userRoles = array_keys($wp_roles->roles);
        }

        add_action('admin_menu', array($this, 'create_menu'));
        add_action('wp_ajax_upload_price_import_file', array($this, 'check_file_uploaded'));
        add_action('wp_ajax_import_prices', array($this, 'import_prices'));
    }

    public function create_menu() 
    {

        add_submenu_page(
            'options-writing.php',
            esc_html__('WooCommerce Price Importer', 'woocommerce-ultimate-pricing'),
            esc_html__('WooCommerce Price Importer', 'woocommerce-ultimate-pricing'),
            'manage_options',
            'woocommerce-ultimate-pricing-importer',
            array($this, 'settings_page')
        );
    }

    public function settings_page() 
    {
    ?>
        <div class="wrap">
            <h1><?php esc_html_e('WooCommerce Ultimate Pricing Importer', 'woocommerce-ultimate-pricing') ?></h1>

            <script>
            (function( $ ) {

                var loadingIcon = '<svg width="40" height="10" viewBox="0 0 120 30" xmlns="http://www.w3.org/2000/svg" fill="#3171ee"> <circle cx="15" cy="15" r="15"> <animate attributeName="r" from="15" to="15" begin="0s" dur="0.8s" values="15;9;15" calcMode="linear" repeatCount="indefinite" /> <animate attributeName="fill-opacity" from="1" to="1" begin="0s" dur="0.8s" values="1;.5;1" calcMode="linear" repeatCount="indefinite" /> </circle> <circle cx="60" cy="15" r="9" fill-opacity="0.3"> <animate attributeName="r" from="9" to="9" begin="0s" dur="0.8s" values="9;15;9" calcMode="linear" repeatCount="indefinite" /> <animate attributeName="fill-opacity" from="0.5" to="0.5" begin="0s" dur="0.8s" values=".5;1;.5" calcMode="linear" repeatCount="indefinite" /> </circle> <circle cx="1005" cy="15" r="15"> <animate attributeName="r" from="15" to="15" begin="0s" dur="0.8s" values="15;9;15" calcMode="linear" repeatCount="indefinite" /> <animate attributeName="fill-opacity" from="1" to="1" begin="0s" dur="0.8s" values="1;.5;1" calcMode="linear" repeatCount="indefinite" /> </circle></svg>';

                jQuery(document).ready(function(e){

                    var ajaxURL = '<?php echo admin_url('admin-ajax.php'); ?>';
                    var logContainer = $('.woocommerce-ultimate-pricing-import-log');
                    var maxRows = 0;
                    var maxColumms = 0;
                    var file = "";
                    var batch = 0;
                    var maxBatches = 1;

                    function importPrices() {

                        $.ajax( {
                            type: "POST",
                            url: ajaxURL,
                            dataType: 'json',
                            data: {
                                'action' : 'import_prices',
                                "batch" : batch,
                                "rows" : maxRows,
                                "update_prices" : $('#price-import-update:checked').val(),
                                "columns" : maxColumms,
                                "file" : file
                            }, 
                            success: function( response ) {

                                logContainer.append(response.message);

                                if(!response.return) {
                                    return;
                                }

                                if(batch == maxBatches) {
                                    $('#price-import-button').html('Import Prices').removeAttr('disabled');
                                    $('#woocommerce-ultimate-pricing-import-current').text(maxRows);
                                    return;
                                }

                                $('#woocommerce-ultimate-pricing-import-current').text( (batch + 1) * 100);

                                batch++;
                                importPrices();
                            }
                        })
                    }

                    // Submit form data via Ajax
                    jQuery("#woocommerce-ultimate-pricing-import").on('submit', function(e) {
                        e.preventDefault();
                        
                        // get file field value using field id
                        var fileInputElement = document.getElementById("price-import-file").files[0];
                        if(!fileInputElement) {
                            alert('File missing');
                            return false;

                        }  else {
                            
                            var fileName = fileInputElement.name;

                            $('#price-import-button').html(loadingIcon).attr('disabled', 'disabled');

                            jQuery.ajax({
                                url: ajaxURL,
                                type: "POST",
                                processData:  false,
                                dataType: 'json',
                                contentType:  false,
                                data:  new FormData(this),
                                success : function( response ){

                                    logContainer.html('<p>' + response.message + '</p>');

                                    maxRows = response.rows;
                                    maxColumms = response.columns;
                                    file = response.file;

                                    if(!response.return) {
                                        alert('File upload error');
                                        return;
                                    }

                                    if(response.return){

                                        maxBatches = Math.ceil( response.rows / 100 ) - 1;
                                        importPrices();
                                    }
                                },
                            });
                            return false;
                        }
                        return false;
                    });
                });
            })( jQuery );
            </script>

            <form id="woocommerce-ultimate-pricing-import" method="post" enctype="multipart/form-data">

                <input type="hidden" name="action" value="upload_price_import_file">
                <input type="hidden" name="price_import_file_path" id="price_import_file_path" value="">

                <table class="form-table">            
                    <tr valign="top">
                        <td colspan="2" width="300px">
                            <b><?php esc_html_e('Batch Import:', 'woocommerce-ultimate-pricing') ?></b> <?php esc_html_e('The importer will import prices in batches of 100 via AJAX. You can leave, but do not close this tab until the import has been finished!', 'woocommerce-ultimate-pricing') ?><br>
                            <b><?php esc_html_e('How to use best:', 'woocommerce-ultimate-pricing') ?></b> <?php esc_html_e('1. Export all Prices (see settings > export prices button) 2. Do modifications in excel file. 3. Import modified Excel file.', 'woocommerce-ultimate-pricing') ?>
                        </td>
                    </tr>                    
                    <tr valign="top">
                        <th scope="row"><?php esc_html_e('File to Import', 'woocommerce-ultimate-pricing') ?></th>
                        <td><input type="file" id="price-import-file" name="price_import_file" accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet, application/vnd.ms-excel"></td>
                    </tr>
                </table>

                <p class="submit">
                    <button id="price-import-button" type="submit" name="submit" id="submit" class="button button-primary"><?php esc_html_e('Import Prices', 'woocommerce-ultimate-pricing') ?></button>
                </p>
            </form>

            <div class="woocommerce-ultimate-pricing-import-log" style="max-height: 300px; overflow: scroll; background-color: #eaeaea; padding: 10px;">
                
            </div>
        </div>
    <?php 
    }

    public function check_file_uploaded()
    {
        if(!isset($_POST) || empty($_POST)) {
            return FALSE;
        }

        if(!isset($_FILES['price_import_file']['name']) || empty($_FILES['price_import_file']['name'])){
            $this->notice = "No file selected.";
            $this->notice(false);
        }

        $xls_mimetypes = array(
                'application/vnd.ms-excel',
                'application/vnd.ms-excel.addin.macroEnabled.12',
                'application/vnd.ms-excel.sheet.binary.macroEnabled.12',
                'application/vnd.ms-excel.sheet.macroEnabled.12',
                'application/vnd.ms-excel.template.macroEnabled.12',
                'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                'application/octet-stream'
        );
        if(!in_array($_FILES['price_import_file']['type'], $xls_mimetypes)){
            $this->notice =  "Only Excels files allowed (given: ".$_FILES["price_import_file"]["type"].")";
            $this->notice(false);
        }

        // Use the wordpress function to upload
        // price_import_file corresponds to the position in the $_FILES array
        // 0 means the content is not associated with any other posts
        $attachment_id = media_handle_upload('price_import_file', 0);
        $file = get_attached_file($attachment_id);

        // Error checking using WP functions
        if(is_wp_error($attachment_id)){
            $this->notice = "Error uploading file: " . $attachment_id->get_error_message();
            $this->notice(false);
        }

        $this->file = $file;

       $writer = 'Xlsx';

        $useExcel2007 = $this->get_option('excel2007');
        if($useExcel2007 == "1") {
            $writer = 'Xls';
        }

        $file = str_replace('avada//avada', 'avada', $file);
        $file = str_replace('//', '/', $file);

        try {
            $objReader = IOFactory::createReader($writer);
            $objReader->setReadDataOnly(true);
            $objPHPExcel = $objReader->load($file);

            $objWorksheet = $objPHPExcel->getActiveSheet();
            $this->rows = $objWorksheet->getHighestRow(); 
            $highestColumn = $objWorksheet->getHighestColumn(); 

            $this->columns = Coordinate::columnIndexFromString($highestColumn);
        } catch (Exception $e) {
            $this->notice = 'Your file seems to be corrupt.<br/>' . $e->getMessage();
            $this->notice(false);
        }

        $this->notice = '<b>File uploaded. Found ' . $this->rows . ' rows and ' . $this->columns . ' columns</b>. Importing prices now: <span id="woocommerce-ultimate-pricing-import-current">0</span> / ' . $this->rows;

        $this->notice();
    }

    public function import_prices()
    {
       
        if(!isset($_POST['file']) || empty($_POST['file'])) {
            $this->notice .= 'File missing.';
            $this->notice(false);
        }

        $file = $_POST['file'];
        $batch = (int) $_POST['batch'];

        $row = ($batch * 100) + 1;
        $rows = ($batch + 1) * 100;

        if($batch == 0) {
            $row = 2;
            $rows = 100;
        }

        // max rows reached
        if($rows > $_POST['rows']) {
            $rows = $_POST['rows'];
        }

        $writer = 'Xlsx';

        $useExcel2007 = $this->get_option('excel2007');
        if($useExcel2007 == "1") {
            $writer = 'Xls';
        }

        $file = str_replace('avada//avada', 'avada', $file);
        $file = str_replace('//', '/', $file);

        try {
            $objReader = IOFactory::createReader($writer);
            $objReader->setReadDataOnly(true);
            $objPHPExcel = $objReader->load($file);

            $objWorksheet = $objPHPExcel->getActiveSheet();

            $highestColumn = $objWorksheet->getHighestColumn(); 
            $highestColumnIndex = Coordinate::columnIndexFromString($highestColumn); // PHPExcel_Cell::columnIndexFromString($highestColumn); 

            $prices = array();
            $keys = array();

            for ($col = 0; $col <= $highestColumnIndex; ++$col) {
                $keys[$col] = $objWorksheet->getCellByColumnAndRow($col, 1)->getValue();
                continue;
            }   

            for ($row; $row <= $rows; $row++) {
                for ($col = 0; $col <= $highestColumnIndex; ++$col) {
                    $prices[$row][$keys[$col]] = $objWorksheet->getCellByColumnAndRow($col, $row)->getValue();
                }   
                $firstLine = false;
            }
            

            if(empty($prices)) {
                $this->notice .= 'No Prices found<br/>';
            }

            $i = 1;
            foreach ($prices as $price) {
                $i++;

                $salePriceExists = false;

                $this->notice .= 'Row ' . $i . ': ';

                if(empty($price['id'])) {
                    $this->notice .= 'Product ID missing!<br/>';
                    continue; 
                }

                $product = wc_get_product($price['id']);
                if(!$product) {
                    $this->notice .= 'Product with ID: ' . $price['id'] . ' not found<br/>';
                    continue;
                }

                if(!isset($price['regular_price'])) {
                    $price['regular_price'] = 0;
                }

                if(!isset($price['sale_price'])) {
                    $price['sale_price'] = 0;
                }

                $regular_price = floatval( str_replace(',', '.', $price['regular_price']) );
                $sale_price = floatval( str_replace(',', '.', $price['sale_price']) );

                // Can be set to 0
                $product->set_regular_price( $regular_price );

                if(isset($sale_price) && !empty($sale_price) ) {

                    if( $sale_price <= $regular_price ) {
                        $salePriceExists = true;
                        $product->set_sale_price( $sale_price);
                        $product->set_price( $sale_price);
                    } else {
                        $this->notice .= 'Sale price can not be higher than regular price. ';
                        $product->set_price( $regular_price);
                    }
                
                } else {
                    $product->set_price( $regular_price);
                }

                $customUserRolePrices = array();
                foreach ($this->userRoles as $userRole) {

                    if(isset($price[$userRole . '_price']) && !empty($price[$userRole . '_price'])) {

                        if($salePriceExists && $price[$userRole . '_price'] >= $sale_price) {
                            $this->notice .= $userRole .' price can not be higher than sale price. ';
                        } elseif($price[$userRole . '_price'] >= $regular_price) {
                            $this->notice .= $userRole .' price can not be higher than regular price. ';
                        } else {
                            $customUserRolePrices[$userRole] = floatval( str_replace(',', '.', $price[$userRole . '_price'] ) );
                        }
                    }
                }

                $customUserRolePrices = array_filter($customUserRolePrices);
                if(!empty($customUserRolePrices)) {
                    $product->update_meta_data( 'woocommerce_ultimate_pricing_prices', $customUserRolePrices);
                } else {
                    $product->delete_meta_data( 'woocommerce_ultimate_pricing_prices');
                }
                
                $save = $product->save();

                if(!$save) {
                    $this->notice .= 'Price: ' . $price['name'] . ' error: <br/>' . $i . '<br/><br/>';
                    continue;
                } else {
                    $this->notice .= 'Product ' . $price['name'] . ' with ID ' . $price['id'] . ' successfully updated<br/>';
                }
            }
        } catch (Exception $e) {
            $this->notice = 'Your file seems to be corrupt.<br/>' . $e->getMessage();
        }

        $this->notice(true);
    }

    public function notice($return = true)
    {
        $response = array(
            'return' => $return,
            'message' => $this->notice,
            'file' => $this->file,
            'rows' => $this->rows,
            'columns' => $this->columns,
        );

        echo json_encode($response);
        die();
    }
}